"""
##### PARAMETERS #####

"""

from playlabs.cli.ansible import Ansible
from playlabs.cli.parser import Parser

import pytest


@pytest.fixture
def p():
    return Parser()


@pytest.fixture
def a(p):
    return Ansible(p)


def test_ansible_sudo(a):
    assert a.sudo([]) == [
        '--become',
        '--become-method', 'sudo',
        '--become-user', 'root'
    ]


def test_ansible_inventory(a):
    a.INVENTORY_FILE = 'filename.yml'
    i = a.inventory()
    assert i == ['--inventory', 'filename.yml']



import os
import shlex
import subprocess
import sys

import pexpect

import sh

import yaml

find = [
    'inventory.yaml',
    'inventory.yml',
    'inventory/inventory.yaml',
    'inventory/inventory.yml',
]


class Ansible(object):
    def __init__(self, parser):
        os.environ.setdefault('ANSIBLE_STDOUT_CALLBACK', 'yaml')
        self.commands = {
            'init': self.init,
            'deploy': self.deploy,
            'install': self.install
        }
        self.parser = parser
        self.vault = Vault()
        self.PLAYBOOKS = os.path.join(
            os.path.dirname(__file__),
            os.pardir
        )
        self.key_path = '.ssh_private_key'

        self.INVENTORY_FILE = None
        self.INVENTORY_DIR = None
        for i in find:
            if os.path.exists(i):
                self.INVENTORY_FILE = os.path.abspath(i)
                self.INVENTORY_DIR = os.path.abspath(
                    os.path.dirname(self.INVENTORY_FILE)
                )
                break

    def sudo(self, options):
        if '--become' not in options:
            options.append('--become')
        if '--become-method' not in options:
            options += [
                '--become-method', 'sudo',
            ]
        if '--become-user' not in options:
            options += [
                '--become-user', 'root'
            ]
        return options

    def inventory(self):
        if self.INVENTORY_FILE:
            return ['--inventory', self.INVENTORY_FILE]
        return []

    def playbook(self, name, args, sudo=True):
        if '--nosudo' in args:
            args.pop(args.index('--nosudo'))
            sudo = False

        if sudo:
            args = self.sudo(args)

        cmd = ['ansible-playbook']
        cmd.append('-v')

        cmd += self.inventory()
        cmd += args
        if os.path.exists(self.key_path):
            cmd += ['--private-key', self.key_path]
        cmd.append(os.path.join(self.PLAYBOOKS, name))
        cmd = [shlex.quote(i) for i in cmd]

        print(' '.join(cmd))
        res = self.spawn(cmd)

        return res

    def spawn(self, cmd):
        child = pexpect.spawn(' '.join(cmd), encoding='utf8', timeout=300)
        if self.parser.password:
            child.expect('SSH password.*')
            child.sendline(self.parser.password)

            if self.parser.user != 'root':
                child.expect('BECOME password.*')
                child.sendline(self.parser.password)
        self.interact(child)
        return child.exitstatus

    def interact(self, child):
        if sys.stdout.isatty():
            child.interact()
            child.wait()
        else:
            while child.isalive():
                for i in child.read(1):
                    print(i, end='', flush=True)

    def prepare_init(self):
        init_options_array = []
        for host in self.parser.hosts:
            options = self.parser.options + [
                '--inventory',
                f'{host},',
                '--limit',
                f'{host}',
            ]
            init_options_array.append(
                [
                    'init.yml',
                    options,
                    self.parser.user != 'root'
                ]
            )
        return init_options_array

    def prepare_install(self):
        install_options_array = []
        for role in self.parser.roles:
            options = self.parser.options + ['-e', f'role={role}']

            if self.parser.hosts:
                options += [
                    '--inventory',
                    ','.join(self.parser.hosts) + ',',
                    '--limit',
                    ','.join(self.parser.hosts) + ',',
                ]
                playbook = 'role-all.yml'
            else:
                playbook = 'role.yml'

            install_options_array.append([playbook, options, True])
        return install_options_array

    def prepare_deploy(self):
        options = self.parser.options
        hosts = self.parser.hosts
        if hosts:
            options += [
                '--inventory',
                ','.join(hosts) + ',',
                '--limit',
                ','.join(hosts) + ',',
            ]
            options += ['-e', f'role=project']
            playbook = 'role-all.yml'
        else:
            playbook = 'project.yml'
        return [[playbook, options, True]]

    def init(self):
        print('Initializing user (no role argument found)')
        return self.compute(self.prepare_init)

    def install(self):
        print(f'Installing roles {",".join(self.parser.roles)}')
        return self.compute(self.prepare_install)

    def deploy(self):
        print(f'Deploying project')
        return self.compute(self.prepare_deploy)

    def compute(self, prepare_func):
        options_array = prepare_func()
        for playbook, options, sudo in options_array:
            retcode = self.playbook(playbook, options, sudo)
            if retcode:
                return retcode

    def package(self, package):
        if sh.which(package):
            return 0

        options = [
            '-c',
            'local',
            '-i',
            'localhost,',
            '-e',
            f'package={package}',
        ]
        user = os.getenv('USER')
        if user != 'root':
            options = self.sudo(options)
        res = self.playbook('package.yml', options)
        if res != 0:
            print('Passing passwords requires sshpass command')
        return res

    def get_ssh_key(self):
        if self.INVENTORY_DIR and self.parser.user:
            inv_key = os.path.join(
                self.INVENTORY_DIR,
                'keys',
                self.parser.user
            )

            if os.path.exists(inv_key):
                print(f'Using {inv_key}')

                with open(inv_key, 'r') as f:
                    content = f.read()

                if 'ANSIBLE_VAULT' in content:
                    decrypted_vault = self.vault.decrypt(inv_key)
                    self.set_ssh_key(decrypted_vault)
                else:
                    self.set_ssh_key(inv_key)

                return

        key = os.getenv('SSH_PRIVATE_KEY')
        if key:
            print('Using SSH_PRIVATE_KEY env var')
            self.set_ssh_key(key)

    def set_ssh_key(self, key):
        with open(self.key_path, 'wb+') as f:
            if isinstance(key, str):
                print('Encoding string key to bytes with unicode')
                key = key.encode('utf8')
            f.write(key)
        os.chmod(self.key_path, 0o700)

    def unset_ssh_key(self):
        if os.path.exists(self.key_path):
            os.unlink(self.key_path)

    def set_sudo(self):
        if self.INVENTORY_DIR is not None:
            users = os.path.join(
                self.INVENTORY_DIR,
                'group_vars/all/users.yml'
            )
            print(f'Looking for users file: {users}')
            if os.path.exists(users):
                with open(users, 'r') as f:
                    users_data = yaml.load(f.read())

                if not self.parser.checksudo(users_data):
                    # detect that deploy user has no sudo
                    self.parser.options.append('--nosudo')


class Vault(object):
    def __init__(self):
        self.pass_file = None
        self.is_temp = False

    def prepare(self):
        self.pass_file = '.vault'

        if 'ANSIBLE_VAULT_PASSWORD_FILE' in os.environ:
            if not os.path.exists(os.getenv('ANSIBLE_VAULT_PASSWORD_FILE')):
                self.pass_file = os.environ.pop(
                    'ANSIBLE_VAULT_PASSWORD_FILE'
                )

        elif 'ANSIBLE_VAULT_PASSWORD' in os.environ:
            with open(self.pass_file, 'w+') as f:
                f.write(os.getenv('ANSIBLE_VAULT_PASSWORD'))
            self.is_temp = True

        if os.path.exists(self.pass_file):
            os.environ['ANSIBLE_VAULT_PASSWORD_FILE'] = self.pass_file

    def clean(self):
        if self.pass_file:
            os.environ['ANSIBLE_VAULT_PASSWORD_FILE'] = self.pass_file

        # todo: make this safer in a try block or context processor or
        # something
        if self.is_temp:
            os.unlink('.vault')

    def decrypt(self, key):
        print('Decrypting with vault')
        return subprocess.check_output([
            'ansible-vault', 'view', key
        ])



import os
import shutil
import subprocess
import sys


class Clicmd(object):
    def __init__(self, parser):
        self.parser = parser
        self.commands = {
            'help': self.help,
            'scaffold': self.scaffold,
            'git': self.git
        }
        self.key_path = '.ssh_private_key'

    def help(self):
        helptext = None
        with open(os.path.join(os.path.dirname(__file__), 'help')) as f:
            helptext = f.read()
        print(helptext)

    def scaffold(self):
        target = self.parser.options[0]

        if os.path.exists(target):
            print(f'Drop existing {target}?')
            if input().lower() in ['y', 'yes']:
                shutil.rmtree(target)
            else:
                print('Aborting')
                sys.exit(1)

        print(f'Provisioning {target} from inventory template')
        shutil.copytree(
            os.path.join(
                os.path.dirname(__file__),
                os.pardir,
                'inventory_template',
            ),
            target,
        )

    def git(self):
        key = os.getenv('SSH_PRIVATE_KEY')
        if key:
            print('Using SSH_PRIVATE_KEY env var')
            with open(self.key_path, 'w+') as f:
                f.write(key)
            os.chmod(self.key_path, 0o600)

            os.environ['GIT_SSH_COMMAND'] = ' '.join((
                f'ssh -i {self.key_path}',
                '-o StrictHostKeyChecking=no'
            ))

        subprocess.check_output(['git'] + self.parser.options)

        if os.path.exists(self.key_path):
            os.unlink(self.key_path)



import sys

from ansible.plugins.callback import CallbackBase


class CallbackModule(CallbackBase):
    """
    This callback module exists non-zero if no hosts match
    """
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = 'aggregate'
    CALLBACK_NAME = 'failer'
    CALLBACK_NEEDS_WHITELIST = False

    def __init__(self):
        super(CallbackModule, self).__init__()

    def v2_runner_on_unreachable(self, result):
        sys.exit(1)

    def v2_playbook_on_no_hosts_matched(self):
        sys.exit(1)

    def v2_playbook_on_stats(self, stats):
        found_stats = False

        for key in ['ok', 'failures', 'dark', 'changed', 'skipped']:
            if len(getattr(stats, key)) > 0:
                found_stats = True
                break

        if not found_stats:
            sys.exit(1)



import collections
import json
import os

from .exception import PlaylabsCliException


class Parser(object):
    def __init__(self):
        self.handles = {
            'install': self.handle_install,
            'deploy': self.handle_deploy,
            'init': self.handle_init,
            'git': self.handle_git,
            'scaffold': self.handle_scaffold,
            'backup': self.handle_backup,
            'restore': self.handle_restore,
            'log': self.handle_log,
            '-i': self.handle_inventory,
            '-p': self.handle_plugins,
        }
        self.argv = []
        self.primary_tokens = self.handles.keys()
        self._action = None
        self._user = None
        self._password = None
        self.roles = []
        self.hosts = []
        self.options = []
        self.subvars = dict()
        self.options_dict = dict()
        self.argcount = 0

    @property
    def action(self):
        return self._action

    @action.setter
    def action(self, name):
        if self._action:
            raise PlaylabsCliException(
                'Only one action can be set:\n' +
                f'Action defined: "{self._action}"\n' +
                f'Current action: "{name}"'
            )
        elif self.password and name != 'init':
            raise PlaylabsCliException(
                'Password only needed for initialization'
            )
        else:
            self._action = name

    @property
    def user(self):
        return self._user

    @user.setter
    def user(self, name):
        if self._user:
            raise PlaylabsCliException('User should only be defined once')
        else:
            self._user = name

    @property
    def password(self):
        return self._password

    @password.setter
    def password(self, value):
        if self.action != 'init':
            raise PlaylabsCliException(
                'Password only needed for initialization'
            )
        else:
            self._password = value

    def handle_scaffold(self):
        self.action = 'scaffold'
        target = self.popargv()
        if target:
            self.options = [target]
        else:
            raise PlaylabsCliException('No target was specified')
        if len(self.argv):
            raise PlaylabsCliException('Too many parameters')

    def handle_git(self):
        self.action = 'git'
        # would be super sweet to prepend git@ to host name in the case of
        # git clone, because that's the username we use on all host git
        # services from github to git.coop
        self.options = self.argv.copy()
        self.argv.clear()

    def handle_install(self):
        self.action = 'install'
        arg = self.popargv()
        if arg:
            self.roles = arg.split(',')
        else:
            raise PlaylabsCliException('No role was specified to install')

    def handle_deploy(self):
        self.action = 'deploy'

    def handle_init(self):
        self.action = 'init'

    def handle_backup(self):
        self.action = 'backup'

    def handle_restore(self):
        self.action = 'restore'

    def handle_log(self):
        self.action = 'log'

    def handle_host(self, arg):
        host = arg.split('@')[-1]
        if host:
            self.hosts.append(host)
        if not arg.startswith('@'):
            left = arg.split('@')[0]
            if ':' in left:
                self.user, self.password = arg.split('@')[0].split(':')
                if '--ask-pass' not in self.options:
                    self.options.append('--ask-pass')

                become = '--ask-become-pass' not in self.options
                if self.user != 'root' and become:
                    self.options.append('--ask-become-pass')
            else:
                self.user = left

        if self.user:
            self.options += ['--user', self.user]

    def handle_inventory(self):
        arg = self.popargv()
        if not arg:
            raise PlaylabsCliException('Inventory: missing parameter')
        for i in arg.split(','):
            if os.path.exists(i):
                self.options += ['-i', i]
            else:
                raise PlaylabsCliException(f'Inventory not found: {i}')
                print(f'command line inventory {i} cannot be found')

    def handle_plugins(self):
        arg = self.popargv()
        if not arg:
            raise PlaylabsCliException('Plugins: missing parameter')
        plugins_path = os.path.join(os.path.dirname(__file__), '../plugins')
        if os.path.exists(plugins_path):
            plugins_list = os.listdir(plugins_path)
            plugins = []
            for p in arg.split(','):
                if p in plugins_list:
                    plugins.append(p)
                else:
                    self.argcount += 1
                    raise PlaylabsCliException(f'Plugin not found: {p}')
            if plugins:
                self.options.append('-e')
                if len(plugins) > 1:
                    self.options.append(f'plugins={",".join(plugins)}')
                else:
                    self.options.append(f'plugins={plugins[0]}')
            else:  # should be useless
                raise PlaylabsCliException(
                    'Plugin: "-p" found but no plugins specified'
                )

    def handle_vars(self, arg):
        if arg == '-e':
            return

        if '=' in arg and not arg.startswith('--'):
            if arg[0] == '=' or arg[-1] == '=':
                raise PlaylabsCliException(
                    f'Wrong variable format {arg}, ' +
                    'variable definition cannot start neither end with "="'
                )
            name = arg.split('=')[0]
            if '.' in name:
                if name[0] == '.' or name[-1] == '.':
                    raise PlaylabsCliException(
                        f'Wrong variable format {name}, ' +
                        'variable name cannot start neither end with "."'
                    )

                def setattribute(a, v):
                    if len(a) > 1:
                        return {a[0]: setattribute(a[1:], v)}
                    else:
                        return {a[0]: v}
                descriptor, val = arg.split('=')
                variable = descriptor.split('.')[0]
                descarray = descriptor.split('.')[1:]
                if variable not in self.subvars.keys():
                    self.subvars[variable] = setattribute(descarray, val)
                else:
                    self.subvars[variable].update(setattribute(descarray, val))
            else:
                self.options += ['-e', arg]
                self.options_dict[name] = arg.split('=')[1]
        else:
            self.options.append(arg)

    def skip(self, arg):
        return

    def ssh_config(self):
        ssh = collections.OrderedDict()

        ssh['ControlMaster'] = 'auto'
        ssh['ControlPersist'] = '60s'
        ssh['ControlPath'] = f'.ssh_control_path_{self.user}'
        ssh['StrictHostKeyChecking'] = 'no'
        self.options += ['--ssh-extra-args', ' '.join([
            f'-o {key}={value}' for key, value in ssh.items()
        ])]

    def popargv(self):
        if len(self.argv):
            self.argcount += 1
            return self.argv.pop(0)

    def parse(self, argv):
        if not argv:
            self.action = 'help'
        else:
            self.argv = argv

        while self.argv:
            arg = self.popargv()

            if arg in self.primary_tokens:
                self.handles[arg]()
            elif arg in ('-u', '--user'):
                self.user = self.popargv()
            elif arg.startswith('-u=') or arg.startswith('--user='):
                self.user = arg.split('=')[-1]
            else:
                if '@' in arg and '=' not in arg:
                    self.handle_host(arg)
                else:
                    self.handle_vars(arg)

        if self.action in ['init', 'install', 'deploy']:
            if not self.user:
                self.user = os.getenv("USER")

            if self.hosts == ['localhost']:
                self.options += ['-c', 'local']
            else:
                self.ssh_config()

            if self.subvars:
                self.options += ['-e', json.dumps(self.subvars)]

        self.print()

    def checksudo(self, data):
        for user in data.get('users'):
            if user['name'] != self.user:
                continue

            roles = user.get('roles', {})
            if 'ssh' in roles:
                if 'sudo' not in roles.get('ssh', []):
                    return False
            else:
                return False
        return True

    def print(self):
        if self.user:
            print(f'Play user: {self.user}')
        if self.hosts:
            print(f'Play hosts: {self.hosts}')
        if self.roles:
            print(f'Play roles: {self.roles}')
        if self.options:
            print(f'Options: {self.options}')



import subprocess

from .exception import PlaylabsCliException


class Ssh(object):
    def __init__(self, parser):
        self.parser = parser
        self.commands = {
            'backup': self.backup,
            'restore': self.restore,
            'log': self.log
        }

    def backup(self):
        if not self.parser.options:
            raise PlaylabsCliException('Backup: Missing container')
        elif len(self.parser.options) > 1:
            raise PlaylabsCliException('Backup: Too many parameters')
        else:
            container = self.parser.options[0]
            subproc_p = [
                'ssh', self.parser.hosts[0],
                f'sudo [ -d /home/{container} ] || exit 1;',
                f'sudo [ -f /home/{container}/backup.sh ] || exit 2;',
                f'sudo /home/{container}/backup.sh',
            ]
            retcode = subprocess.call(subproc_p)
            if retcode == 1:
                raise PlaylabsCliException(
                    f'Backup: container {container} not found'
                )
            elif retcode == 2:
                raise PlaylabsCliException(
                    f'Backup: restore.sh not found'
                )

    def restore(self):
        if not self.parser.options:
            raise PlaylabsCliException('Restore: Missing container')
        else:
            container = self.parser.options[0]
            subproc_p = [
                'ssh', self.parser.hosts[0],
                f'sudo [ -d /home/{container} ] || exit 1;',
                f'sudo [ -f /home/{container}/restore.sh ] || exit 2;',
                f'sudo /home/{container}/restore.sh',
            ] + self.parser.options[1:]
            retcode = subprocess.call(subproc_p)
            if retcode == 1:
                raise PlaylabsCliException(
                    f'Restore: container {container} not found'
                )
            elif retcode == 2:
                raise PlaylabsCliException(
                    f'Restore: restore.sh not found'
                )

    def log(self):
        if not self.parser.options:
            raise PlaylabsCliException('Log: Missing container')
        elif len(self.parser.options) > 1:
            raise PlaylabsCliException('Log: Too many parameters')
        else:
            container = self.parser.options[0]
            subproc_p = [
                'ssh', self.parser.hosts[0],
                f'sudo [ -d /home/{container} ] || exit 1;',
                f'sudo docker logs -f {container}'
            ]
            retcode = subprocess.call(subproc_p)
            if retcode:
                raise PlaylabsCliException(
                    f'Log: container {container} not found'
                )



"""
##### ERROR CASES #####

 user cannot be defined twice
 vars cannot start with '.'
 nothing can start with '='
 plugins must exist
 plugins must be specified
 inventory file must exist
 inventory must be specified

[LATER] host must be found either in CLI or in inventories
"""

from playlabs.cli.parser import Parser

import pytest


@pytest.fixture
def p():
    return Parser()


def test_multi_user_error(p):
    with pytest.raises(Exception):
        p.parse(['-u', 'user1', '-u', 'user2'])


def test_multi_user_error_at(p):
    with pytest.raises(Exception):
        p.parse(['-u', 'user1', 'user@host'])


def test_var_starting_with_eq(p):
    with pytest.raises(Exception):
        p.parse(['=value'])


def test_var_ending_with_eq(p):
    with pytest.raises(Exception):
        p.parse(['var='])


def test_var_starting_with_dot(p):
    with pytest.raises(Exception):
        p.parse(['.var=value'])


def test_var_ending_with_dot(p):
    with pytest.raises(Exception):
        p.parse(['var.=value'])


def test_plugin_must_exits(p):
    with pytest.raises(Exception):
        p.parse(['-p', 'notaplugin'])


def test_plugin_must_be_specified(p):
    with pytest.raises(Exception):
        p.parse(['-p'])


def test_inventory_must_exist(p):
    with pytest.raises(Exception):
        p.parse(['-i', 'inexistant13245.yml'])


def test_inventory_must_be_specified(p):
    with pytest.raises(Exception):
        p.parse(['-i'])



import os

from ansible.errors import AnsibleError
from ansible.plugins.lookup import LookupBase

try:
    from __main__ import display
except ImportError:
    from ansible.utils.display import Display
    display = Display()


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        for term in terms:
            display.vv("File lookup term: %s" % term)
            if not os.path.exists(term):
                raise AnsibleError("could locate file in lookup: %s" % term)
        return True



#!/usr/bin/python
import os
import subprocess

from ansible import errors


class FilterModule(object):
    def filters(self):
        return {
            'docker_env_dict': self.docker_env_dict,
            'exists': self.exists,
            'key_pub_exists': self.key_pub_exists,
            'key_pub_read': self.key_pub_read,
            'vaulted_password': self.vaulted_password,
            'vaulted_read': self.vaulted_read,
        }

    def key_pub_exists(self, name):
        return os.path.exists(f'keys/{name}.pub')

    def key_pub_read(self, name):
        with open(f'keys/{name}.pub', 'r') as f:
            return f.read()

    def exists(self, path):
        return os.path.exists(path)

    def docker_env_dict(self, env):
        return {
            i.split('=')[0]: i.split('=')[1]
            for i in env
        }

    def vaulted_read(self, path, default=None):
        if not os.path.exists(path):
            if default is not None:
                return default
            raise errors.AnsibleFilterError(
                f'{path} not found and no default provided')
        out = subprocess.check_output([
            'ansible-vault',
            'view',
            path,
        ])
        return out.decode('utf8').strip()

    def vaulted_password(self, name, default=None):
        path = os.path.join('passwords/', name)
        return self.vaulted_read(path, default)



import os
import sys

import sh

from .ansible import Ansible
from .clicmd import Clicmd
from .exception import PlaylabsCliException
from .parser import Parser
from .ssh import Ssh

LOCAL_BIN = f'{os.getenv("HOME")}/.local/bin'
LOCAL_BIN_PLAYLABS = f'{LOCAL_BIN}/playlabs'
BASH_PROFILE = f'{os.getenv("HOME")}/.bash_profile'

if '/' in sys.argv[0]:
    os.environ['PATH'] = ':'.join([
        os.path.dirname(sys.argv[0]),
        os.getenv('PATH')
    ])


def cli():  # noqa
    parser = Parser()
    try:
        parser.parse(sys.argv[1:])
    except PlaylabsCliException as e:
        sys.argv[parser.argcount] += '<---'
        print(f'\nError parsing command line input: {e}\n')
        print(' '.join(sys.argv) + '\n\n')

    # todo: check if connection works with provided credentials if any
    # otherwise try without credentials, and strip them from now on
    # because that would mean that the host is already initped
    # meanwhile, it fails with root@ ... Permission denied because
    # we disable root login in ssh role

    clicmd = Clicmd(parser)
    ansible = Ansible(parser)
    ssh = Ssh(parser)

    retcode = 0
    if parser.password and not sh.which('sshpass'):
        retcode = ansible.package('sshpass')
        if retcode:
            print('Error installing sshpass')

    ansible.vault.prepare()
    ansible.get_ssh_key()
    ansible.set_sudo()

    # we have ssh options working for that no ?
    # for host in parser.hosts:
    #    print(f'Adding {host} to ~/.ssh/known_hosts')
    #    known_host(host)

    def clean():
        if not os.getenv('PLAYLABS_NOCLEAN'):
            ansible.unset_ssh_key()
            ansible.vault.clean()

    for module in [clicmd, ansible, ssh]:
        if parser.action in module.commands.keys():
            try:
                retcode = module.commands[parser.action]()
            except PlaylabsCliException as e:
                print(e)
                clean()
                sys.exit(1)

            if retcode:
                break

    clean()
    sys.exit(retcode)



"""
##### PARAMETERS #####

 test -u,--user username[:password], -u=username, --user=username
 test default_user
 test [username[:password]]@host
 test localhost options
 test ssh_config
 test vars, subvars, ansible options
 test roles
 test inventories
 test plugins
"""

import json
import os

from playlabs.cli.parser import Parser

import pytest


@pytest.fixture
def p():
    return Parser()


def test_parser_class_exists(p):
    assert p


def test_user_name_shortopt(p):
    p.parse(['init', '-u', 'testname'])
    assert p.user == 'testname'


def test_user_name_longopt(p):
    p.parse(['init', '--user', 'testname2'])
    assert p.user == 'testname2'


def test_user_name_shortvar(p):
    p.parse(['init', '-u=testname1'])
    assert p.user == 'testname1'


def test_user_name_longvar(p):
    p.parse(['init', '--user=testname2'])
    assert p.user == 'testname2'


def test_user_default(p):
    p.parse(['init'])
    assert p.user == os.getenv('USER')


def test_at_host_only(p):
    p.parse(['init', '@testhost'])
    assert p.hosts == ['testhost']


def test_user_at_host(p):
    p.parse(['init', 'testname3@testhost2'])
    assert p.user == 'testname3'
    assert p.hosts == ['testhost2']


def test_user_pass_at_host(p):
    p.parse(['init', 'testname4:password@testhost3'])
    assert p.user == 'testname4'
    assert p.hosts == ['testhost3']
    assert p.password == 'password'
    assert '--ask-become-pass' in p.options
    assert '--ask-pass' in p.options


def test_root_pass_at_host(p):
    p.parse(['init', 'root:password@testhost3'])
    assert p.user == 'root'
    assert p.hosts == ['testhost3']
    assert p.password == 'password'
    assert '--ask-become-pass' not in p.options
    assert '--ask-pass' in p.options


def test_localhost(p):
    p.parse(['init', '@localhost'])
    assert '-c' in p.options
    assert 'local' in p.options
    assert p.options.index('local') == p.options.index('-c') + 1


def test_ssh_config(p):
    p.parse(['init', '@host'])
    assert '--ssh-extra-args' in p.options


def test_vars(p):
    p.parse([
        'init', '--extra=opt',
        'extra-opt2',
        'extra=var'
    ])
    assert 'extra-opt2' in p.options
    assert '--extra=opt' in p.options
    assert p.options[p.options.index('--extra=opt') - 1] != '-e'
    assert 'extra=var' in p.options
    assert p.options[p.options.index('extra=var') - 1] == '-e'


def test_subvars(p):
    p.parse(['init', 'env.testsub=value'])
    assert '-e' in p.options
    assert json.dumps({'env': {'testsub': 'value'}}) in p.options


def test_deep_vars(p):
    p.parse(['init', 'test.very.deep.var=deep'])
    assert json.dumps({
        'test': {
            'very': {
                'deep': {'var': 'deep'}
            }}}) in p.options


def test_merge_vars(p):
    p.parse(['init', 'test.sub=value1', 'test.sub2=value2'])
    assert json.dumps({
        'test': {
            'sub': 'value1',
            'sub2': 'value2'
        }
    }) in p.options


def test_role_uniq(p):
    p.parse(['install', 'rolename'])
    assert 'rolename' in p.roles


def test_role_multi(p):
    roles = 'nginx,docker,firewall'
    p.parse(['install', roles])
    assert p.roles == roles.split(',')


def test_plugin_uniq(p):
    p.parse(['init', '-p', 'django'])
    assert 'plugins=django' in p.options


def test_plugin_multi(p):
    plugins = 'django,postgres,sentry'
    p.parse(['init', '-p', plugins])
    assert f'plugins={plugins}' in p.options
    assert p.options[p.options.index(f'plugins={plugins}') - 1] == '-e'


def test_inventory_uniq(p):
    dirpath = os.path.join(os.path.dirname(__file__), '..')
    invpath = os.path.join(dirpath, 'inventory_template/inventory.yaml')
    p.parse(['init', '-i', invpath])
    assert invpath in p.options
    assert p.options[p.options.index(invpath) - 1] == '-i'


def test_inventory_multi(p):
    dirpath = os.path.join(os.path.dirname(__file__), '..')
    invpath1 = os.path.join(dirpath, 'inventory_template/inventory.yaml')
    invpath2 = os.path.join(dirpath, 'init.yml')
    p.parse(['init', '-i', ','.join([invpath1, invpath2])])
    assert invpath1 in p.options
    assert p.options[p.options.index(invpath1) - 1] == '-i'
    assert invpath2 in p.options
    assert p.options[p.options.index(invpath2) - 1] == '-i'



import pip
from setuptools import setup, find_packages, Command
import os, sys
from subprocess import Popen, PIPE
import shutil

with open('requirements.txt') as reqs_file:
    install_reqs = reqs_file.readlines()

home_path = os.getenv('HOME')
bashcompletion_dir = home_path
for d in ['.local', 'share', 'bash-completion', 'completions']:
    bashcompletion_dir = os.path.join(bashcompletion_dir, d)
    if not os.path.exists(bashcompletion_dir):
        os.mkdir(bashcompletion_dir)

bashcompletion_path = os.path.join(bashcompletion_dir, 'playlabs')
shutil.copyfile('bash-completion.sh', bashcompletion_path)


VERSION = '0.3.0'


setup(
    name='playlabs',
    version=VERSION if '@' not in VERSION else 'dev',
    description='The obscene ansible paas distribution',
    author='James Pic',
    author_email='jamespic@gmail.com',
    url='https://yourlabs.io/oss/playlabs',
    packages=['playlabs'],
    include_package_data=True,
    license='MIT',
    install_requires=install_reqs,
    entry_points={
        'console_scripts': [
            'playlabs = playlabs.cli.main:cli',
        ],
    },
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



import os
import re
import subprocess

BASH_PROFILE = f'{os.getenv("HOME")}/.bash_profile'


def patch():
    with open(BASH_PROFILE, 'a+') as f:
        f.write(
            'export PATH=$PATH:$HOME/.local/bin # playlabs'
        )


def known_host(target):
    ssh = f'{os.getenv("HOME")}/.ssh'
    if not os.path.exists(ssh):
        os.makedirs(ssh)
        os.chmod(ssh, 0o700)

    known_hosts = f'{ssh}/known_hosts'
    skip = False
    if os.path.exists(known_hosts):
        with open(known_hosts, 'r') as f:
            for l in f.readlines():
                if re.match('^' + target + ' ', l):
                    skip = True

    if not skip:
        key = subprocess.check_output(['ssh-keyscan', target])
        with open(known_hosts, 'ab+') as f:
            f.write(key)
        os.chmod(known_hosts, 0o600)
